@@ -16,7 +16,6 @@
 """Storage backend for SWIFT"""
 
 import hashlib
-import httplib
 import logging
 import math
 
@@ -24,9 +23,9 @@ from oslo_config import cfg
 from oslo_utils import excutils
 from oslo_utils import units
 import six
-import six.moves.urllib.parse as urlparse
+from six.moves import http_client
+from six.moves import urllib
 import swiftclient
-import urllib
 
 import glance_store
 from glance_store._drivers.swift import utils as sutils
@@ -127,6 +126,18 @@ _SWIFT_OPTS = [
 
 
 def swift_retry_iter(resp_iter, length, store, location, context):
+    if not length and isinstance(resp_iter, six.BytesIO):
+        if six.PY3:
+            # On Python 3, io.BytesIO does not have a len attribute, instead
+            # go the end using seek to get the size of the file
+            pos = resp_iter.tell()
+            resp_iter.seek(0, 2)
+            length = resp_iter.tell()
+            resp_iter.seek(pos)
+        else:
+            # On Python 2, StringIO has a len attribute
+            length = resp_iter.len
+
     length = length if length else (resp_iter.len
                                     if hasattr(resp_iter, 'len') else 0)
     retries = 0
@@ -195,7 +206,8 @@ class StoreLocation(location.StoreLocation):
 
     def _get_credstring(self):
         if self.user and self.key:
-            return '%s:%s' % (urllib.quote(self.user), urllib.quote(self.key))
+            return '%s:%s' % (urllib.parse.quote(self.user),
+                              urllib.parse.quote(self.key))
         return ''
 
     def get_uri(self, credentials_included=True):
@@ -270,12 +282,12 @@ class StoreLocation(location.StoreLocation):
                 raise exceptions.BadStoreUri(message=reason)
             key = cred_parts.pop()
             user = ':'.join(cred_parts)
-            creds = urllib.unquote(creds)
+            creds = urllib.parse.unquote(creds)
             try:
                 self.user, self.key = creds.rsplit(':', 1)
             except exceptions.BadStoreConfiguration:
-                self.user = urllib.unquote(user)
-                self.key = urllib.unquote(key)
+                self.user = urllib.parse.unquote(user)
+                self.key = urllib.parse.unquote(key)
         else:
             self.user = None
             self.key = None
@@ -319,7 +331,7 @@ class StoreLocation(location.StoreLocation):
             LOG.info(_LI("Invalid store URI: %(reason)s"), {'reason': reason})
             raise exceptions.BadStoreUri(message=reason)
 
-        pieces = urlparse.urlparse(uri)
+        pieces = urllib.parse.urlparse(uri)
         assert pieces.scheme in ('swift', 'swift+http', 'swift+https',
                                  'swift+config')
 
@@ -418,10 +430,10 @@ class BaseStore(driver.Store):
 
         try:
             resp_headers, resp_body = connection.get_object(
-                container=location.container, obj=location.obj,
+                location.container, location.obj,
                 resp_chunk_size=self.CHUNKSIZE, headers=headers)
         except swiftclient.ClientException as e:
-            if e.http_status == httplib.NOT_FOUND:
+            if e.http_status == http_client.NOT_FOUND:
                 msg = _("Swift could not find object %s.") % location.obj
                 LOG.warn(msg)
                 raise exceptions.NotFound(message=msg)
@@ -440,7 +452,7 @@ class BaseStore(driver.Store):
         class ResponseIndexable(glance_store.Indexable):
             def another(self):
                 try:
-                    return self.wrapped.next()
+                    return next(self.wrapped)
                 except StopIteration:
                     return ''
 
@@ -456,7 +468,7 @@ class BaseStore(driver.Store):
             connection = self.get_connection(location, context=context)
         try:
             resp_headers = connection.head_object(
-                container=location.container, obj=location.obj)
+                location.container, location.obj)
             return int(resp_headers.get('content-length', 0))
         except Exception:
             return 0
@@ -577,7 +589,7 @@ class BaseStore(driver.Store):
                 # Now we write the object manifest and return the
                 # manifest's etag...
                 manifest = "%s/%s-" % (location.container, location.obj)
-                headers = {'ETag': hashlib.md5("").hexdigest(),
+                headers = {'ETag': hashlib.md5(b"").hexdigest(),
                            'X-Object-Manifest': manifest}
 
                 # The ETag returned for the manifest is actually the
@@ -602,7 +614,7 @@ class BaseStore(driver.Store):
             return (location.get_uri(credentials_included=include_creds),
                     image_size, obj_etag, {})
         except swiftclient.ClientException as e:
-            if e.http_status == httplib.CONFLICT:
+            if e.http_status == http_client.CONFLICT:
                 msg = _("Swift already has an image at this location")
                 raise exceptions.Duplicate(message=msg)
 
@@ -630,7 +642,7 @@ class BaseStore(driver.Store):
                 dlo_manifest = headers.get('x-object-manifest')
                 slo_manifest = headers.get('x-static-large-object')
             except swiftclient.ClientException as e:
-                if e.http_status != httplib.NOT_FOUND:
+                if e.http_status != http_client.NOT_FOUND:
                     raise
 
             if _is_slo(slo_manifest):
@@ -662,7 +674,7 @@ class BaseStore(driver.Store):
             connection.delete_object(location.container, location.obj)
 
         except swiftclient.ClientException as e:
-            if e.http_status == httplib.NOT_FOUND:
+            if e.http_status == http_client.NOT_FOUND:
                 msg = _("Swift could not find image at URI.")
                 raise exceptions.NotFound(message=msg)
             else:
@@ -679,7 +691,7 @@ class BaseStore(driver.Store):
         try:
             connection.head_container(container)
         except swiftclient.ClientException as e:
-            if e.http_status == httplib.NOT_FOUND:
+            if e.http_status == http_client.NOT_FOUND:
                 if self.conf.glance_store.swift_store_create_container_on_put:
                     try:
                         msg = (_LI("Creating swift container %(container)s") %
@@ -888,7 +900,7 @@ class MultiTenantStore(BaseStore):
         try:
             connection.post_container(location.container, headers=headers)
         except swiftclient.ClientException as e:
-            if e.http_status == httplib.NOT_FOUND:
+            if e.http_status == http_client.NOT_FOUND:
                 msg = _("Swift could not find image at URI.")
                 raise exceptions.NotFound(message=msg)
             else:
